/**
 * Copyright 2014 Daum Kakao Corp.
 *
 * Redistribution and modification in source or binary forms are not permitted without specific prior written permission. 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kakao.auth.callback;

import com.kakao.auth.ErrorCode;
import com.kakao.auth.ErrorResult;

/**
 * @author leoshin, created at 15. 8. 4..
 */
public abstract class ResponseCallback<T> {

    /**
     * 세션이 닫혀서 또는 카카오스토리 사용자가 아닌 경우를 제외한 이유로 요청이 실패한 경우 호출된다.
     * 아래 에러 종류에 따라 적절한 처리를 한다.
     * {@link ErrorCode#INVALID_PARAM_CODE}, <br/>
     * {@link ErrorCode#INVALID_SCOPE_CODE}, <br/>
     * {@link ErrorCode#NOT_SUPPORTED_API_CODE}, <br/>
     * {@link ErrorCode#EXCEED_MAX_UPLOAD_SIZE}, <br/>
     * {@link ErrorCode#EXECUTION_TIMED_OUT}, <br/>
     * {@link ErrorCode#EXCEED_MAX_UPLOAD_NUMBER}, <br/>
     * {@link ErrorCode#INVALID_STORY_SCRAP_URL}, <br/>
     * {@link ErrorCode#INVALID_STORY_ACTIVITY_ID}, <br/>
     * {@link ErrorCode#INTERNAL_ERROR_CODE}, <br/>
     * {@link ErrorCode#CLIENT_ERROR_CODE}, <br/>
     * {@link ErrorCode#EXCEED_LIMIT_CODE}, <br/>
     * {@link ErrorCode#KAKAO_MAINTENANCE_CODE}, <br/>
     * @param errorResult 실패한 원인이 담긴 결과
     */
    public abstract void onFailure(final ErrorResult errorResult);

    /**
     * request에 대한 result.
     * @param result nullable value
     */
    public abstract void onSuccess(T result);

    /**
     * 해당 Job이 시작될때 불린다.
     * Appilcation 에서 프로그래스바를 보여줄때 사용가능합니다.
     */
    public void onDidStart() {
    }

    /**
     * 해당 Job이 끝날때 불린다.
     * Appilcation 에서 프로그래스바를 닫을때 사용가능합니다.
     */
    public void onDidEnd() {
    }

    /**
     * 요청한 Request가 실패했을때 불리는 callback.
     * 해당 Error를 통해서 Request별로 공통처리 및 onFailure Callback을 재정의 할 수 있습니다.
     * @param errorResult 실패결과.
     */
    public void onFailureForUiThread(ErrorResult errorResult) {
        onFailure(errorResult);
    }

    /**
     * 요청한 Request가 성공했을때 불리는 callback.
     * 해당 callback을 통해서 Request별로 공통처리 및 onSuccess Callback을 재정의 할 수 있습니다.
     * @param result 성공에 대한 response.
     */
    public void onSuccessForUiThread(T result) {
        onSuccess(result);
    }
}
